@@ -84,3 +84,16 @@ absinfo_normalize(const struct input_absinfo *abs)
 {
 	return absinfo_normalize_value(abs, abs->value);
 }
+
+static inline double
+absinfo_scale_axis(const struct input_absinfo *absinfo, double val, double to_range)
+{
+	return (val - absinfo->minimum) * to_range / absinfo_range(absinfo);
+}
+
+static inline double
+absinfo_convert_to_mm(const struct input_absinfo *absinfo, double v)
+{
+	double value = v - absinfo->minimum;
+	return value/absinfo->resolution;
+}
