@@ -137,6 +137,14 @@ struct litest_device_interface {
 	struct input_event *pad_ring_change_events;
 	struct input_event *pad_ring_end_events;
 
+	/**
+	 * Pad events, LITEST_AUTO_ASSIGN is allowed on event values
+	 * for ABS_WHEEL
+	 */
+	struct input_event *pad_dial_start_events;
+	struct input_event *pad_dial_change_events;
+	struct input_event *pad_dial_end_events;
+
 	/**
 	 * Pad events, LITEST_AUTO_ASSIGN is allowed on event values
 	 * for ABS_RX
