@@ -39,7 +39,7 @@ enum hold_gesture_behaviour {
 };
 
 static void
-test_gesture_swipe_3fg(int cardinal, enum hold_gesture_behaviour hold)
+test_gesture_swipe_3fg(enum cardinal cardinal, enum hold_gesture_behaviour hold)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
@@ -150,7 +150,7 @@ test_gesture_swipe_3fg(int cardinal, enum hold_gesture_behaviour hold)
 }
 
 static void
-test_gesture_swipe_4fg(int cardinal, enum hold_gesture_behaviour hold)
+test_gesture_swipe_4fg(enum cardinal cardinal, enum hold_gesture_behaviour hold)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
@@ -288,7 +288,7 @@ test_gesture_swipe_4fg(int cardinal, enum hold_gesture_behaviour hold)
 }
 
 static void
-test_gesture_pinch_2fg(int cardinal, enum hold_gesture_behaviour hold)
+test_gesture_pinch_2fg(enum cardinal cardinal, enum hold_gesture_behaviour hold)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
@@ -411,7 +411,7 @@ test_gesture_pinch_2fg(int cardinal, enum hold_gesture_behaviour hold)
 }
 
 static void
-test_gesture_pinch_3fg(int cardinal, enum hold_gesture_behaviour hold)
+test_gesture_pinch_3fg(enum cardinal cardinal, enum hold_gesture_behaviour hold)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
@@ -526,7 +526,7 @@ test_gesture_pinch_3fg(int cardinal, enum hold_gesture_behaviour hold)
 }
 
 static void
-test_gesture_pinch_4fg(int cardinal, enum hold_gesture_behaviour hold)
+test_gesture_pinch_4fg(enum cardinal cardinal, enum hold_gesture_behaviour hold)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
@@ -648,7 +648,7 @@ test_gesture_pinch_4fg(int cardinal, enum hold_gesture_behaviour hold)
 }
 
 static void
-test_gesture_spread(int cardinal, enum hold_gesture_behaviour hold)
+test_gesture_spread(enum cardinal cardinal, enum hold_gesture_behaviour hold)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
@@ -946,7 +946,7 @@ END_TEST
 
 START_TEST(gestures_swipe_3fg)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_swipe_3fg(cardinal, HOLD_GESTURE_IGNORE);
 }
 END_TEST
@@ -958,7 +958,7 @@ START_TEST(gestures_swipe_3fg_btntool)
 	struct libinput_event *event;
 	struct libinput_event_gesture *gevent;
 	double dx, dy;
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	double dir_x, dir_y;
 	int cardinals[NCARDINALS][2] = {
 		{ 0, 30 },
@@ -1106,7 +1106,7 @@ END_TEST
 
 START_TEST(gestures_swipe_4fg)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_swipe_4fg(cardinal, HOLD_GESTURE_IGNORE);
 }
 END_TEST
@@ -1118,7 +1118,7 @@ START_TEST(gestures_swipe_4fg_btntool)
 	struct libinput_event *event;
 	struct libinput_event_gesture *gevent;
 	double dx, dy;
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	double dir_x, dir_y;
 	int cardinals[NCARDINALS][2] = {
 		{ 0, 30 },
@@ -1216,28 +1216,28 @@ END_TEST
 
 START_TEST(gestures_pinch)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_pinch_2fg(cardinal, HOLD_GESTURE_IGNORE);
 }
 END_TEST
 
 START_TEST(gestures_pinch_3fg)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_pinch_3fg(cardinal, HOLD_GESTURE_IGNORE);
 }
 END_TEST
 
 START_TEST(gestures_pinch_4fg)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_pinch_4fg(cardinal, HOLD_GESTURE_IGNORE);
 }
 END_TEST
 
 START_TEST(gestures_spread)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_spread(cardinal, HOLD_GESTURE_IGNORE);
 }
 END_TEST
@@ -1483,7 +1483,7 @@ START_TEST(gestures_hold)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
-	int nfingers = _i; /* ranged test */
+	int nfingers = litest_test_param_get_i32(test_env->params, "fingers");
 
 	litest_disable_tap(dev->libinput_device);
 	litest_drain_events(li);
@@ -1496,7 +1496,7 @@ START_TEST(gestures_hold_tap_enabled)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
-	int nfingers = _i; /* ranged test */
+	int nfingers = litest_test_param_get_i32(test_env->params, "fingers");
 
 	litest_enable_tap(dev->libinput_device);
 	litest_drain_events(li);
@@ -1509,7 +1509,7 @@ START_TEST(gestures_hold_cancel)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
-	int nfingers = _i; /* ranged test */
+	int nfingers = litest_test_param_get_i32(test_env->params, "fingers");
 
 	litest_disable_tap(dev->libinput_device);
 	litest_drain_events(li);
@@ -1522,7 +1522,7 @@ START_TEST(gestures_hold_cancel_tap_enabled)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
-	int nfingers = _i; /* ranged test */
+	int nfingers = litest_test_param_get_i32(test_env->params, "fingers");
 
 	litest_enable_tap(dev->libinput_device);
 	litest_drain_events(li);
@@ -1533,42 +1533,42 @@ END_TEST
 
 START_TEST(gestures_hold_then_swipe_3fg)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_swipe_3fg(cardinal, HOLD_GESTURE_REQUIRE);
 }
 END_TEST
 
 START_TEST(gestures_hold_then_swipe_4fg)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_swipe_4fg(cardinal, HOLD_GESTURE_REQUIRE);
 }
 END_TEST
 
 START_TEST(gestures_hold_then_pinch_2fg)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_pinch_2fg(cardinal, HOLD_GESTURE_REQUIRE);
 }
 END_TEST
 
 START_TEST(gestures_hold_then_pinch_3fg)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_pinch_3fg(cardinal, HOLD_GESTURE_REQUIRE);
 }
 END_TEST
 
 START_TEST(gestures_hold_then_pinch_4fg)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_pinch_4fg(cardinal, HOLD_GESTURE_REQUIRE);
 }
 END_TEST
 
 START_TEST(gestures_hold_then_spread)
 {
-	int cardinal = _i; /* ranged test */
+	enum cardinal cardinal = litest_test_param_get_i32(test_env->params, "direction");
 	test_gesture_spread(cardinal, HOLD_GESTURE_REQUIRE);
 }
 END_TEST
@@ -1629,7 +1629,7 @@ START_TEST(gestures_hold_once_tap_n_drag)
 {
 	struct litest_device *dev = litest_current_device();
 	struct libinput *li = dev->libinput;
-	int nfingers = _i; /* ranged test */
+	int nfingers = litest_test_param_get_i32(test_env->params, "fingers");
 	unsigned int button = 0;
 
 	if (nfingers > litest_slot_count(dev))
@@ -1782,24 +1782,491 @@ START_TEST(gestures_hold_and_motion_after_timeout)
 }
 END_TEST
 
+START_TEST(gestures_3fg_drag)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	uint32_t finger_count;
+	bool tap_enabled;
+	litest_test_param_fetch(test_env->params,
+				"fingers", 'u', &finger_count,
+				"tap-enabled", 'b', &tap_enabled);
+
+	if (litest_slot_count(dev) < 3)
+		return LITEST_NOT_APPLICABLE;
+
+	if (libinput_device_config_3fg_drag_get_finger_count(dev->libinput_device) < (int)finger_count)
+		return LITEST_NOT_APPLICABLE;
+
+	litest_enable_3fg_drag(dev->libinput_device, finger_count);
+	if (tap_enabled)
+		litest_enable_tap(dev->libinput_device);
+	else
+		litest_disable_tap(dev->libinput_device);
+
+	litest_drain_events(li);
+
+	double y = 30.0;
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_down(dev, i, 10 + i, y);
+
+	litest_dispatch(li);
+
+	litest_drain_events_of_type(li, LIBINPUT_EVENT_GESTURE_HOLD_BEGIN, LIBINPUT_EVENT_GESTURE_HOLD_END);
+
+	if (tap_enabled) {
+		litest_checkpoint("Expecting no immediate button press as tapping is enabled");
+		litest_assert_empty_queue(li);
+	} else {
+		litest_checkpoint("Expecting immediate button press as tapping is disabled");
+		litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	}
+
+	while (y < 60.0) {
+		y += 2;
+		for (uint32_t i = 0; i < finger_count; i++)
+			litest_touch_move(dev, i, 10 + i, y);
+		litest_dispatch(li);
+	}
+
+	if (tap_enabled) {
+		litest_checkpoint("Expecting late button press as tapping is enabled");
+		litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	}
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_up(dev, i);
+
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_timeout_3fg_drag();
+	litest_dispatch(li);
+
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+}
+END_TEST
+
+START_TEST(gestures_3fg_drag_lock_resume_3fg_motion)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	uint32_t finger_count;
+	bool tap_enabled;
+	bool wait_for_timeout;
+	litest_test_param_fetch(test_env->params,
+				"fingers", 'u', &finger_count,
+				"tap-enabled", 'b', &tap_enabled,
+				"wait", 'b', &wait_for_timeout);
+
+	if (litest_slot_count(dev) < 3)
+		return LITEST_NOT_APPLICABLE;
+
+	if (libinput_device_config_3fg_drag_get_finger_count(dev->libinput_device) < (int)finger_count)
+		return LITEST_NOT_APPLICABLE;
+
+	litest_enable_3fg_drag(dev->libinput_device, finger_count);
+	if (tap_enabled)
+		litest_enable_tap(dev->libinput_device);
+	else
+		litest_disable_tap(dev->libinput_device);
+
+	litest_drain_events(li);
+
+	litest_checkpoint("Putting three fingers down + movement)");
+	double y = 30.0;
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_down(dev, i, 10 + i, y);
+
+	litest_dispatch(li);
+
+	while (y < 60.0) {
+		y += 2;
+		for (uint32_t i = 0; i < finger_count; i++)
+			litest_touch_move(dev, i, 10 + i, y);
+		litest_dispatch(li);
+	}
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	litest_checkpoint("Releasing all fingers");
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_up(dev, i);
+
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_checkpoint("Putting three fingers down (no movement)");
+	y = 30.0;
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_down(dev, i, 10 + i, y);
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_dispatch(li);
+
+	litest_checkpoint("Waiting past finger switch timeout");
+	litest_timeout_finger_switch();
+	litest_dispatch(li);
+
+	if (wait_for_timeout) {
+		litest_checkpoint("Waiting past tap/3fg drag timeout");
+		litest_timeout_3fg_drag();
+		litest_dispatch(li);
+		litest_assert_empty_queue(li);
+	}
+
+	litest_checkpoint("Moving three fingers");
+	while (y < 60.0) {
+		y += 2;
+		for (uint32_t i = 0; i < finger_count; i++)
+			litest_touch_move(dev, i, 10 + i, y);
+		litest_dispatch(li);
+	}
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	litest_checkpoint("Releasing three fingers");
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_up(dev, i);
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_timeout_3fg_drag();
+	litest_dispatch(li);
+
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+}
+END_TEST
+
+START_TEST(gestures_3fg_drag_lock_resume_3fg_release_no_motion)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	uint32_t finger_count;
+	bool tap_enabled;
+	bool wait_for_timeout;
+	litest_test_param_fetch(test_env->params,
+				"fingers", 'u', &finger_count,
+				"tap-enabled", 'b', &tap_enabled,
+				"wait", 'b', &wait_for_timeout);
+
+	/* tap-enabled for 4fg finger count doesn't make a difference */
+	bool expect_tap = finger_count <= 3 && tap_enabled && !wait_for_timeout;
+
+	if (litest_slot_count(dev) < 3)
+		return LITEST_NOT_APPLICABLE;
+
+	if (libinput_device_config_3fg_drag_get_finger_count(dev->libinput_device) < (int)finger_count)
+		return LITEST_NOT_APPLICABLE;
+
+	litest_enable_3fg_drag(dev->libinput_device, finger_count);
+
+	if (tap_enabled)
+		litest_enable_tap(dev->libinput_device);
+	else
+		litest_disable_tap(dev->libinput_device);
+
+	litest_drain_events(li);
+
+	litest_checkpoint("Putting 3 fingers down with motion for drag");
+	double y = 30.0;
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_down(dev, i, 10 + i, y);
+
+	litest_dispatch(li);
+	while (y < 60.0) {
+		y += 2;
+		for (uint32_t i = 0; i < finger_count; i++)
+			litest_touch_move(dev, i, 10 + i, y);
+		litest_dispatch(li);
+	}
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_up(dev, i);
+
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_checkpoint("Putting 3 fingers down again (no motion)");
+	y = 30.0;
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_down(dev, i, 10 + i, y);
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_checkpoint("Waiting past finger switch timeout");
+	litest_timeout_finger_switch();
+	litest_dispatch(li);
+
+	if (wait_for_timeout) {
+		litest_checkpoint("Waiting past tap/3fg drag timeout");
+		litest_timeout_3fg_drag();
+		litest_dispatch(li);
+		litest_assert_empty_queue(li);
+	}
+
+	litest_checkpoint("Releasing three fingers");
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_up(dev, i);
+	litest_dispatch(li);
+
+	if (expect_tap) {
+		/* If we're not waiting and tapping is enabled, this is
+		 * the equivalent of a 3fg tap within the drag timeout */
+		litest_checkpoint("Expecting 3fg drag release");
+		litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+		litest_checkpoint("Expecting 3fg tap");
+		litest_assert_button_event(li, BTN_MIDDLE, LIBINPUT_BUTTON_STATE_PRESSED);
+		litest_assert_button_event(li, BTN_MIDDLE, LIBINPUT_BUTTON_STATE_RELEASED);
+	}
+
+	litest_assert_empty_queue(li);
+	litest_timeout_3fg_drag();
+	litest_dispatch(li);
+
+	if (!expect_tap)
+		litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+
+	litest_assert_empty_queue(li);
+}
+END_TEST
+
+START_TEST(gestures_3fg_drag_lock_resume_1fg_motion)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	uint32_t finger_count;
+	bool tap_enabled;
+	litest_test_param_fetch(test_env->params,
+				"fingers", 'u', &finger_count,
+				"tap-enabled", 'b', &tap_enabled);
+
+	if (litest_slot_count(dev) < 3)
+		return LITEST_NOT_APPLICABLE;
+
+	if (libinput_device_config_3fg_drag_get_finger_count(dev->libinput_device) < (int)finger_count)
+		return LITEST_NOT_APPLICABLE;
+
+	litest_enable_3fg_drag(dev->libinput_device, finger_count);
+
+	if (tap_enabled)
+		litest_enable_tap(dev->libinput_device);
+	else
+		litest_disable_tap(dev->libinput_device);
+
+	litest_drain_events(li);
+
+	litest_checkpoint("Putting 3 fingers down + motion to trigger 3fg drag");
+	double y = 30.0;
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_down(dev, i, 10 + i, y);
+
+	while (y < 60.0) {
+		y += 2;
+		for (uint32_t i = 0; i < finger_count; i++)
+			litest_touch_move(dev, i, 10 + i, y);
+		litest_dispatch(li);
+	}
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	litest_checkpoint("Releasing 3 fingers");
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_up(dev, i);
+
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_checkpoint("Putting 1 finger down and moving it");
+	/* fingers are up, now let's put one finger down and move it */
+	y = 30.0;
+	litest_touch_down(dev, 0, 10, y);
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	/* We need to wait until the gesture code accepts this is one finger only */
+	litest_timeout_finger_switch();
+	litest_dispatch(li);
+
+	while (y < 60.0) {
+		y += 2;
+		litest_touch_move(dev, 0, 10, y);
+		litest_dispatch(li);
+	}
+
+	litest_checkpoint("Expecting drag button release and motion");
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	litest_touch_up(dev, 0);
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_timeout_3fg_drag();
+	litest_dispatch(li);
+}
+END_TEST
+
+START_TEST(gestures_3fg_drag_lock_resume_2fg_scroll)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	uint32_t finger_count;
+	bool tap_enabled;
+	litest_test_param_fetch(test_env->params,
+				"fingers", 'u', &finger_count,
+				"tap-enabled", 'b', &tap_enabled);
+
+	if (litest_slot_count(dev) < 3)
+		return LITEST_NOT_APPLICABLE;
+
+	if (libinput_device_config_3fg_drag_get_finger_count(dev->libinput_device) < (int)finger_count)
+		return LITEST_NOT_APPLICABLE;
+
+	litest_enable_3fg_drag(dev->libinput_device, finger_count);
+
+	if (tap_enabled)
+		litest_enable_tap(dev->libinput_device);
+	else
+		litest_disable_tap(dev->libinput_device);
+
+	litest_drain_events(li);
+
+	litest_checkpoint("Putting 3 fingers down + motion to trigger 3fg drag");
+	double y = 30.0;
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_down(dev, i, 10 + i, y);
+
+	while (y < 60.0) {
+		y += 2;
+		for (uint32_t i = 0; i < finger_count; i++)
+			litest_touch_move(dev, i, 10 + i, y);
+		litest_dispatch(li);
+	}
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	litest_checkpoint("Releasing 3 fingers");
+	for (uint32_t i = 0; i < finger_count; i++)
+		litest_touch_up(dev, i);
+
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_checkpoint("Putting 2 fingers down and moving them");
+	y = 30.0;
+	litest_touch_down(dev, 0, 10, y);
+	litest_touch_down(dev, 1, 20, y);
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_timeout_finger_switch();
+	litest_dispatch(li);
+
+	while (y < 60.0) {
+		y += 2;
+		litest_touch_move(dev, 0, 10, y);
+		litest_touch_move(dev, 1, 20, y);
+		litest_dispatch(li);
+	}
+
+	litest_checkpoint("Expecting drag button release and scroll");
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_only_axis_events(li, LIBINPUT_EVENT_POINTER_SCROLL_FINGER);
+
+	litest_touch_up(dev, 0);
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_timeout_3fg_drag();
+	litest_dispatch(li);
+}
+END_TEST
+
+START_TEST(gestures_3fg_drag_lock_resume_1fg_tap)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+	int finger_count = litest_test_param_get_i32(test_env->params, "fingers");
+
+	if (litest_slot_count(dev) < 3)
+		return LITEST_NOT_APPLICABLE;
+
+	if (libinput_device_config_3fg_drag_get_finger_count(dev->libinput_device) < finger_count)
+		return LITEST_NOT_APPLICABLE;
+
+	litest_enable_3fg_drag(dev->libinput_device, finger_count);
+	litest_enable_tap(dev->libinput_device);
+
+	litest_drain_events(li);
+
+	litest_checkpoint("Putting 3 fingers down for 3fg drag");
+	double y = 30.0;
+	for (int i = 0; i < finger_count; i++)
+		litest_touch_down(dev, i, 10 + i, y);
+
+	litest_dispatch(li);
+
+	while (y < 60.0) {
+		y += 2;
+		for (int i = 0; i < finger_count; i++)
+			litest_touch_move(dev, i, 10 + i, y);
+		litest_dispatch(li);
+	}
+	litest_drain_events_of_type(li,
+				    LIBINPUT_EVENT_GESTURE_HOLD_BEGIN,
+				    LIBINPUT_EVENT_GESTURE_HOLD_END,
+				    -1);
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	litest_checkpoint("Releasing 3 fingers");
+	for (int i = 0; i < finger_count; i++)
+		litest_touch_up(dev, i);
+
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+
+	litest_checkpoint("Tapping with 1 finger");
+	/* fingers are up, now let's tap with one finger */
+	y = 30.0;
+	litest_touch_down(dev, 0, 10, y);
+	litest_dispatch(li);
+	litest_assert_empty_queue(li);
+	litest_touch_up(dev, 0);
+	litest_dispatch(li);
+
+	litest_timeout_tap();
+	litest_dispatch(li);
+
+	litest_checkpoint("Expecting drag release followed by 1fg tap");
+
+	/* 3fg drag lock must be cancelled */
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+	/* And a 1fg tap */
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_empty_queue(li);
+
+	litest_timeout_3fg_drag();
+	litest_dispatch(li);
+}
+END_TEST
+
 TEST_COLLECTION(gestures)
 {
-	struct range cardinals = { N, N + NCARDINALS };
-	struct range range_hold = { 1, 5 };
-	struct range range_multifinger_tap = {1, 4};
-
 	litest_add(gestures_cap, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH);
 	litest_add(gestures_nocap, LITEST_ANY, LITEST_TOUCHPAD);
 
-	litest_add_ranged(gestures_swipe_3fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_swipe_3fg_btntool, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
 	litest_add(gestures_swipe_3fg_btntool_pinch_like, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH);
-	litest_add_ranged(gestures_swipe_4fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_swipe_4fg_btntool, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_pinch, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_pinch_3fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_pinch_4fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_spread, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
 
 	litest_add(gestures_3fg_buttonarea_scroll, LITEST_CLICKPAD, LITEST_SINGLE_TOUCH);
 	litest_add(gestures_3fg_buttonarea_scroll_btntool, LITEST_CLICKPAD, LITEST_SINGLE_TOUCH);
@@ -1812,24 +2279,71 @@ TEST_COLLECTION(gestures)
 	litest_add(gestures_hold_config_is_available, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH|LITEST_SEMI_MT);
 	litest_add(gestures_hold_config_is_not_available, LITEST_TOUCHPAD|LITEST_SEMI_MT, LITEST_ANY);
 
-	litest_add_ranged(gestures_hold, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &range_hold);
-	litest_add_ranged(gestures_hold_tap_enabled, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &range_hold);
-	litest_add_ranged(gestures_hold_cancel, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &range_hold);
-	litest_add_ranged(gestures_hold_cancel_tap_enabled, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &range_hold);
-	litest_add_ranged(gestures_hold_then_swipe_3fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_hold_then_swipe_4fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_hold_then_pinch_2fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_hold_then_pinch_3fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_hold_then_pinch_4fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
-	litest_add_ranged(gestures_hold_then_spread, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, &cardinals);
+	litest_with_parameters(params, "fingers", 'i', 4, 1, 2, 3, 4) {
+		litest_add_parametrized(gestures_hold, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_hold_tap_enabled, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_hold_cancel, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_hold_cancel_tap_enabled, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+	}
+
+	litest_with_parameters(params, "direction", 'I', 8, litest_named_i32(N), litest_named_i32(NE),
+							    litest_named_i32(E), litest_named_i32(SE),
+							    litest_named_i32(S), litest_named_i32(SW),
+							    litest_named_i32(W), litest_named_i32(NW)) {
+		litest_add_parametrized(gestures_swipe_3fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_swipe_3fg_btntool, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_swipe_4fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_swipe_4fg_btntool, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_pinch, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_pinch_3fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_pinch_4fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_spread, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+
+		litest_add_parametrized(gestures_hold_then_swipe_3fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_hold_then_swipe_4fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_hold_then_pinch_2fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_hold_then_pinch_3fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_hold_then_pinch_4fg, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_hold_then_spread, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+	}
+
 	litest_add(gestures_hold_then_3fg_buttonarea_scroll, LITEST_CLICKPAD, LITEST_SINGLE_TOUCH);
 
 	litest_add(gestures_hold_once_on_double_tap, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH);
-	litest_add_ranged(gestures_hold_once_tap_n_drag, LITEST_TOUCHPAD, LITEST_ANY, &range_multifinger_tap);
+	litest_with_parameters(params, "fingers", 'i', 3, 1, 2, 3) {
+		litest_add_parametrized(gestures_hold_once_tap_n_drag, LITEST_TOUCHPAD, LITEST_ANY, params);
+	}
 
 	litest_add(gestures_hold_and_motion_before_timeout, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH);
 	litest_add(gestures_hold_and_motion_after_timeout, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH);
 
+	{
+		struct litest_parameters *params = litest_parameters_new("fingers", 'u', 2, 3, 4,
+									 "tap-enabled", 'b');
+		litest_add_parametrized(gestures_3fg_drag, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_parameters_unref(params);
+	}
+
+	{
+		struct litest_parameters *params = litest_parameters_new("fingers", 'u', 2, 3, 4,
+									 "tap-enabled", 'b',
+									 "wait", 'b');
+		litest_add_parametrized(gestures_3fg_drag_lock_resume_3fg_motion, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_3fg_drag_lock_resume_3fg_release_no_motion, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_parameters_unref(params);
+	}
+
+	{
+		struct litest_parameters *params = litest_parameters_new("fingers", 'u', 2, 3, 4,
+									 "tap-enabled", 'b');
+		litest_add_parametrized(gestures_3fg_drag_lock_resume_1fg_motion, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_add_parametrized(gestures_3fg_drag_lock_resume_2fg_scroll, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+		litest_parameters_unref(params);
+	}
+	litest_with_parameters(params, "fingers", 'i', 2, 3, 4) {
+		litest_add_parametrized(gestures_3fg_drag_lock_resume_1fg_tap, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH, params);
+	}
+
 	/* Timing-sensitive test, valgrind is too slow */
 	if (!RUNNING_ON_VALGRIND)
 		litest_add(gestures_swipe_3fg_unaccel, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH);
