@@ -220,7 +220,6 @@ START_TEST(path_create_pathmax_file)
 }
 END_TEST
 
-
 START_TEST(path_create_destroy)
 {
 	struct libinput *li;
@@ -481,7 +480,7 @@ START_TEST(path_device_sysname)
 	litest_assert_notnull(sysname);
 	litest_assert_int_gt(strlen(sysname), 1U);
 	litest_assert(strchr(sysname, '/') == NULL);
-	litest_assert(strneq(sysname, "event", 5));
+	litest_assert(strstartswith(sysname, "event"));
 
 	libinput_event_destroy(ev);
 }
