@@ -381,9 +381,15 @@ handle_libinput_events(struct context *ctx)
 		case LIBINPUT_EVENT_TABLET_TOOL_AXIS:
 			handle_tablet_axis_event(ctx, ev);
 			break;
-		case LIBINPUT_EVENT_TABLET_TOOL_PROXIMITY:
+		case LIBINPUT_EVENT_TABLET_TOOL_PROXIMITY: {
+			struct libinput_event_tablet_tool *tev =
+				libinput_event_get_tablet_tool_event(ev);
+			struct libinput_tablet_tool *tool =
+				libinput_event_tablet_tool_get_tool(tev);
+			tools_tablet_tool_apply_config(tool, &options);
 			handle_tablet_proximity_event(ctx, ev);
 			break;
+		}
 		case LIBINPUT_EVENT_TABLET_TOOL_TIP:
 			handle_tablet_tip_event(ctx, ev);
 			break;
@@ -555,6 +561,12 @@ main(int argc, char **argv)
 			backend = BACKEND_UDEV;
 			seat_or_device[0] = optarg;
 			break;
+		default:
+			if (tools_parse_option(c, optarg, &options) != 0) {
+				usage();
+				return EXIT_INVALID_USAGE;
+			}
+			break;
 		}
 
 	}
